@@ -6,7 +6,6 @@
 
 #undef PACKAGE
 #undef VERSION
-#undef USE_XIM
-#undef USE_GDKPIXBUF
+#undef USE_GTK24
 
 #undef TMAID_BUILD_ENVIRONMENT
