@@ -17,7 +17,6 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 #include "abort.h"
-#include "charlist.h"
 #include "command.h"
 #include "edit.h"
 #include "file.h"
@@ -34,6 +33,7 @@
 #include "misc/fileio.h"
 #include "misc/misc.h"
 #include "misc/misc_conf.h"
+#include "misc/misc_mdi.h"
 #ifdef G_OS_WIN32
 # include <gdk/gdkwin32.h>
 # include <tchar.h>
@@ -67,7 +67,7 @@ command_open_ok (GtkWidget *widget,
                                                 (GTK_FILE_SELECTION (dialog)));
   gtk_widget_destroy (dialog);
 }
-#endif /* ! defined G_OS_WIN32 && ! defined USE_GTK24 */
+#endif /* not defined G_OS_WIN32 and not defined USE_GTK24 */
 
 
 void
@@ -75,10 +75,13 @@ command_open (gpointer   callback_data,
               guint      callback_action,
               GtkWidget *widget)
 {
-  gchar *file = NULL;
+  gchar *file = NULL, *path;
+#if defined USE_GTK24 || defined G_OS_WIN32
+  gint i;
+#endif /* USE_GTK24 or G_OS_WIN32 */
 #ifdef G_OS_WIN32
   gchar *filter = NULL;
-  gint i, length, pos = 0;
+  gint length, pos = 0;
   LPTSTR lpszPath, lpszFilter;
   OPENFILENAME ofn;
   TCHAR szFile[MAX_PATH];
@@ -119,7 +122,7 @@ command_open (gpointer   callback_data,
   ofn.lStructSize = sizeof (OPENFILENAME);
   ofn.hwndOwner = GDK_WINDOW_HWND (window->window);
   ofn.lpstrFilter = lpszFilter;
-  ofn.nFilterIndex = 1;
+  ofn.nFilterIndex = findex - 1;
   ofn.lpstrCustomFilter = NULL;
   ofn.lpstrFile = szFile;
   ofn.nMaxFile = MAX_PATH;
@@ -137,6 +140,7 @@ command_open (gpointer   callback_data,
           file = g_filename_from_utf8 (utf16file, -1, NULL, NULL, NULL);
           g_free (utf16file);
         }
+      findex = ofn.nFilterIndex - 1;
     }
 # else /* not UNICODE */
     file = g_strdup (szFile);
@@ -147,6 +151,8 @@ command_open (gpointer   callback_data,
   GtkWidget *dialog;
 
 # ifdef USE_GTK24
+  GtkFileFilter *filter;
+
   dialog = gtk_file_chooser_dialog_new (_("Open"),
                         GTK_WINDOW (window), GTK_FILE_CHOOSER_ACTION_OPEN,
                         GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
@@ -154,8 +160,29 @@ command_open (gpointer   callback_data,
                         NULL);
   if (open_path)
     gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), open_path);
+  /* ja:フィルタの設定 */
+  for (i = 0; i < ftnum; i++)
+    {
+      gchar **ext;
+      gint j;
+
+      filter = gtk_file_filter_new ();
+      gtk_file_filter_set_name (filter, ftype[i].text);
+      ext = g_strsplit (ftype[i].ext, G_SEARCHPATH_SEPARATOR_S, G_MAXINT);
+      for (j = 0; ext[j]; j++)
+        gtk_file_filter_add_pattern (filter, ext[j]);
+      g_strfreev (ext);
+      gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
+      if (i == findex)
+        gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter);
+      g_object_set_data (G_OBJECT (filter), "user_data", (gpointer)i);
+    }
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
-    file = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+    {
+      file = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+      filter = gtk_file_chooser_get_filter (GTK_FILE_CHOOSER (dialog));
+      findex = (gint)g_object_get_data (G_OBJECT (filter), "user_data");
+    }
   gtk_widget_destroy (dialog);
 # else /* not USE_GTK24 */
   dialog = gtk_file_selection_new (_("Open"));
@@ -168,7 +195,11 @@ command_open (gpointer   callback_data,
             "clicked", G_CALLBACK (gtk_widget_destroy), GTK_OBJECT (dialog));
   g_object_set_data (G_OBJECT (dialog), "user_data", &file);
   if (open_path)
-    gtk_file_selection_set_filename (GTK_FILE_SELECTION (dialog), open_path);
+    {
+      path = g_strconcat (open_path, G_SEARCHPATH_SEPARATOR_S, NULL);
+      gtk_file_selection_set_filename (GTK_FILE_SELECTION (dialog), path);
+      g_free (path);
+    }
   gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (dialog));
   gtk_widget_show (dialog);
   gtk_grab_add (dialog);
@@ -178,8 +209,9 @@ command_open (gpointer   callback_data,
   if (file)
     {
       g_free (open_path);
-      open_path = fileio_get_full_path (file);
-      *(gchar *)g_basename (open_path) = '\0';
+      path = fileio_get_full_path (file);
+      open_path = g_path_get_dirname (path);
+      g_free (path);
       file_open_edit (file);
       g_free (file);
     }
@@ -191,30 +223,31 @@ command_save (gpointer   callback_data,
               guint      callback_action,
               GtkWidget *widget)
 {
+  gint page_num;
   TmaidWindow *tmaid;
 
-  tmaid = g_object_get_data (G_OBJECT
-                (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
-  if (tmaid->create)
+  page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
+  tmaid = misc_mdi_get_data (MISC_MDI (mdi), page_num);
+  if (misc_mdi_get_created (MISC_MDI (mdi), page_num))
     {
       command_saveas (callback_data, callback_action, widget);
     }
   else
     {
       gchar *text, *utf8str;
+      const gchar *file;
 
-      utf8str = g_filename_to_utf8 (tmaid->file, -1, NULL, NULL, NULL);
+      file = misc_mdi_get_file (MISC_MDI (mdi), page_num);
+      utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
       text = g_strdup_printf (_("%s\nThis file is existed.\n\nOver write?"),
                                                                     utf8str);
       g_free (utf8str);
-      if ((!tmaid->ft.overwrite || !fileio_isfile (tmaid->file)
+      if ((!tmaid->ft.overwrite || !fileio_isfile (file)
                                     || misc_message_box ("Text maid",text, 1,
                                             _("_Yes"), _("_No"), NULL) == 0)
-                                        && file_save_text (tmaid->file, tmaid))
+                                            && file_save_text (file, tmaid))
         {
-          tmaid_change_edit (tmaid, FALSE);
+          misc_mdi_set_edited (MISC_MDI (mdi), page_num, FALSE);
           if (delete_list (&tmaid->undo) + delete_list (&tmaid->redo) > 0)
             set_menu_bar (tmaid);
         }
@@ -227,10 +260,13 @@ command_saveas (gpointer   callback_data,
                 guint      callback_action,
                 GtkWidget *widget)
 {
-  gchar *file = NULL;
+  gchar *file = NULL, *path;
+#if defined USE_GTK24 || defined G_OS_WIN32
+  gint i;
+#endif /* USE_GTK24 or G_OS_WIN32 */
 #ifdef G_OS_WIN32
   gchar *filter = NULL;
-  gint i, length, pos = 0;
+  gint length, pos = 0;
   LPTSTR lpszPath, lpszFilter;
   OPENFILENAME ofn;
   TCHAR szFile[MAX_PATH];
@@ -271,7 +307,7 @@ command_saveas (gpointer   callback_data,
   ofn.lStructSize = sizeof (OPENFILENAME);
   ofn.hwndOwner = GDK_WINDOW_HWND (window->window);
   ofn.lpstrFilter = lpszFilter;
-  ofn.nFilterIndex = 0;
+  ofn.nFilterIndex = findex - 1;
   ofn.lpstrCustomFilter = NULL;
   ofn.lpstrFile = szFile;
   ofn.nMaxFile = MAX_PATH;
@@ -289,6 +325,7 @@ command_saveas (gpointer   callback_data,
           file = g_filename_from_utf8 (utf16file, -1, NULL, NULL, NULL);
           g_free (utf16file);
         }
+      findex = ofn.nFilterIndex - 1;
     }
 # else /* not UNICODE */
     file = g_strdup (szFile);
@@ -299,6 +336,8 @@ command_saveas (gpointer   callback_data,
   GtkWidget *dialog;
 
 # ifdef USE_GTK24
+  GtkFileFilter *filter;
+
   dialog = gtk_file_chooser_dialog_new (_("Save As"),
                         GTK_WINDOW (window), GTK_FILE_CHOOSER_ACTION_SAVE,
                         GTK_STOCK_SAVE_AS, GTK_RESPONSE_ACCEPT,
@@ -306,8 +345,30 @@ command_saveas (gpointer   callback_data,
                         NULL);
   if (save_path)
     gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), save_path);
+  /* ja:フィルタの設定 */
+  for (i = 0; i < ftnum; i++)
+    {
+      gchar **ext;
+      gint j;
+      GtkFileFilter *filter;
+
+      filter = gtk_file_filter_new ();
+      gtk_file_filter_set_name (filter, ftype[i].text);
+      ext = g_strsplit (ftype[i].ext, G_SEARCHPATH_SEPARATOR_S, G_MAXINT);
+      for (j = 0; ext[j]; j++)
+        gtk_file_filter_add_pattern (filter, ext[j]);
+      g_strfreev (ext);
+      gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
+      if (i == findex)
+        gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter);
+      g_object_set_data (G_OBJECT (filter), "user_data", (gpointer)i);
+    }
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
-    file = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+    {
+      file = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+      filter = gtk_file_chooser_get_filter (GTK_FILE_CHOOSER (dialog));
+      findex = (gint)g_object_get_data (G_OBJECT (filter), "user_data");
+    }
   gtk_widget_destroy (dialog);
 # else /* not USE_GTK24 */
   dialog = gtk_file_selection_new (_("Save As"));
@@ -320,7 +381,11 @@ command_saveas (gpointer   callback_data,
             "clicked", G_CALLBACK (gtk_widget_destroy), GTK_OBJECT (dialog));
   g_object_set_data (G_OBJECT (dialog), "user_data", &file);
   if (save_path)
-    gtk_file_selection_set_filename (GTK_FILE_SELECTION (dialog), save_path);
+    {
+      path = g_strconcat (save_path, G_SEARCHPATH_SEPARATOR_S, NULL);
+      gtk_file_selection_set_filename (GTK_FILE_SELECTION (dialog), path);
+      g_free (path);
+    }
   gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (dialog));
   gtk_widget_show (dialog);
   gtk_grab_add (dialog);
@@ -329,13 +394,12 @@ command_saveas (gpointer   callback_data,
 #endif /* not G_OS_WIN32 */
   if (file)
     {
-      gchar *text, *label, *utf8str;
-      GtkWidget *child;
+      gchar *text, *utf8str;
+      gint page_num;
       TmaidWindow *tmaid;
 
-      child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)));
-      tmaid = g_object_get_data (G_OBJECT (child), "user_data");
+      page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
+      tmaid = misc_mdi_get_data (MISC_MDI (mdi), page_num);
       utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
       text = g_strdup_printf (_("%s\nThis file is existed.\n\nOver write?"),
                                                                     utf8str);
@@ -345,28 +409,20 @@ command_saveas (gpointer   callback_data,
                                             _("_Yes"), _("_No"), NULL) == 0)
                                             && file_save_text (file, tmaid))
         {
-          g_free(save_path);
-          save_path = fileio_get_full_path (file);
-          *(gchar *)g_basename (save_path) = '\0';
-          /* ja:ファイル名を調べる */
-          file_delete_edit (tmaid->file);
-          tmaid->file[0] = '\0';
-          label = file_add_edit (file, &tmaid->same);
-          gtk_label_set_text (GTK_LABEL (tmaid->label), label);
-          g_free (label);
-          /* ja:ウインドウメニュー */
-          gtk_label_set_text (GTK_LABEL (gtk_bin_get_child
-                                        (GTK_BIN (tmaid->menu_item))), file);
+          g_free (save_path);
+          path = fileio_get_full_path (file);
+          save_path = g_path_get_dirname (path);
+          g_free (path);
           /* ja:諸設定 */
-          tmaid->create = FALSE;
-          tmaid_change_edit (tmaid, FALSE);
-          g_free (tmaid->file);
-          tmaid->file = g_strdup (file);
+          misc_mdi_set_created (MISC_MDI (mdi), page_num, FALSE);
+          misc_mdi_set_edited (MISC_MDI (mdi), page_num, FALSE);
+          misc_mdi_set_file (MISC_MDI (mdi), page_num, file);
           delete_list (&tmaid->undo);
           delete_list (&tmaid->redo);
           set_menu_bar (tmaid);
-          file_set_history (tmaid->file);
+          file_set_history (file);
         }
+      g_free (text);
       g_free (file);
   }
 }
@@ -378,16 +434,16 @@ command_reload (gpointer   callback_data,
                 GtkWidget *widget)
 {
   gchar *text;
+  const gchar *file;
+  gint page_num;
   LineBuffer *p;
   TmaidWindow *tmaid;
 
-  tmaid = g_object_get_data (G_OBJECT
-                (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
-  text = tmaid->edit ? g_strdup_printf
-                        (_("File %s was edited.\nRestore to existed file?"),
-                                                        tmaid->file) : NULL;
+  page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
+  tmaid = misc_mdi_get_data (MISC_MDI (mdi), page_num);
+  file = misc_mdi_get_file (MISC_MDI (mdi), page_num);
+  text = misc_mdi_get_edited (MISC_MDI (mdi), page_num) ? g_strdup_printf
+            (_("File %s was edited.\nRestore to existed file?"), file) : NULL;
   if (reload_dialog (&tmaid->ft, text))
     {
       /* ja:バッファを解放する */
@@ -406,11 +462,11 @@ command_reload (gpointer   callback_data,
       tmaid->cursor.x = tmaid->cursor.y = tmaid->top.x = tmaid->top.y = 0;
       tmaid->select.x = -1;
       /* ja:再読み込み */
-      file_open_text (tmaid);
+      file_open_text (file, tmaid);
       /* ja:右マージン */
       if (tmaid->ft.limit)
         modify_margin (tmaid);
-      tmaid_change_edit (tmaid, FALSE);
+      misc_mdi_set_edited (MISC_MDI (mdi), page_num, FALSE);
       misc_set_scroll_bar (tmaid->hscroll,
                 G_CALLBACK (signal_value_changed_hscroll), tmaid,
                 0, edit_get_width_max (tmaid) + 1,
@@ -437,10 +493,7 @@ command_print (gpointer   callback_data,
 {
   TmaidWindow *tmaid;
 
-  tmaid = g_object_get_data (G_OBJECT
-                (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
+  tmaid = misc_mdi_get_data (MISC_MDI (mdi), -1);
   printer.select = tmaid->select.x >= 0;
   printer.column = tmaid->ft.margin;
   printer.font_width = tmaid->font_width;
@@ -455,15 +508,10 @@ command_property (gpointer   callback_data,
                   guint      callback_action,
                   GtkWidget *widget)
 {
-  gint i;
   FileType ft;
-  GList *glist;
   TmaidWindow *tmaid;
 
-  tmaid = g_object_get_data (G_OBJECT
-                (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
+  tmaid = misc_mdi_get_data (MISC_MDI (mdi), -1);
   g_memset (&ft, 0, sizeof (FileType));
   property_copy (&ft, &tmaid->ft);
   ft.charset = g_strdup (tmaid->ft.charset);
@@ -478,19 +526,19 @@ command_property (gpointer   callback_data,
         }
       else
         {
+          gint i;
+
           /* ja:同一ファイルタイプをすべて変更 */
-          tmaid->ft.ft_id=ft.ft_id;
+          tmaid->ft.ft_id = ft.ft_id;
           for (i = 0; i < ftnum; i++)
             if (ftype[i].ft_id == ft.ft_id)
               {
                 property_copy (ftype + i, &ft);
                 break;
               }
-          glist = gtk_container_children (GTK_CONTAINER (notebook));
-          for (i = g_list_length (glist) - 1; i >= 0; i--)
+          for (i = misc_mdi_get_n_pages (MISC_MDI (mdi)) - 1; i >= 0; i--)
             {
-              tmaid = g_object_get_data
-                        (G_OBJECT (g_list_nth_data (glist, i)), "user_data");
+              tmaid = misc_mdi_get_data (MISC_MDI (mdi), i);
               if (tmaid->ft.ft_id == ft.ft_id)
                 {
                   g_free (tmaid->ft.charset);
@@ -498,7 +546,6 @@ command_property (gpointer   callback_data,
                   property_change (tmaid, &ft);
                 }
             }
-          g_list_free(glist);
         }
       charlist_renewal_all (ft.charset, ft.ft_id);
   }
@@ -513,15 +560,10 @@ command_close (gpointer   callback_data,
                GtkWidget *widget)
 {
   gint page;
-  TmaidWindow *tmaid;
 
-  if ((page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook))) >= 0)
-    {
-      tmaid = g_object_get_data (G_OBJECT (gtk_notebook_get_nth_page
-                                (GTK_NOTEBOOK (notebook), page)), "user_data");
-      if (prompt_close (tmaid))
-        gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page);
-    }
+  if ((page = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi))) >= 0
+    && prompt_close ((TmaidWindow *)misc_mdi_get_data (MISC_MDI (mdi), page)))
+    gtk_notebook_remove_page (GTK_NOTEBOOK (mdi), page);
 }
 
 
@@ -544,13 +586,8 @@ command_undo (gpointer   callback_data,
               guint      callback_action,
               GtkWidget *widget)
 {
-  TmaidWindow *tmaid;
-
-  tmaid = g_object_get_data (G_OBJECT
-                (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
-  history_operation (tmaid, FALSE);
+  history_operation ((TmaidWindow *)misc_mdi_get_data (MISC_MDI (mdi), -1),
+                                                                        FALSE);
 }
 
 
@@ -559,13 +596,8 @@ command_redo (gpointer   callback_data,
               guint      callback_action,
               GtkWidget *widget)
 {
-  TmaidWindow *tmaid;
-
-  tmaid = g_object_get_data (G_OBJECT
-                (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
-  history_operation (tmaid, TRUE);
+  history_operation ((TmaidWindow *)misc_mdi_get_data (MISC_MDI (mdi), -1),
+                                                                        TRUE);
 }
 
 
@@ -584,17 +616,15 @@ command_copy (gpointer   callback_data,
               guint      callback_action,
               GtkWidget *widget)
 {
-  gint length;
-  TmaidWindow *tmaid;
 
   if (gtk_selection_owner_set (window, GDK_SELECTION_CLIPBOARD,
                                                             GDK_CURRENT_TIME))
     {
-      tmaid = g_object_get_data (G_OBJECT
-                (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
-      g_free(clipboard_text);
+      gint length;
+      TmaidWindow *tmaid;
+
+      tmaid = misc_mdi_get_data (MISC_MDI (mdi), -1);
+      g_free (clipboard_text);
       length = edit_get_sel_bytes (tmaid, &tmaid->select, &tmaid->cursor);
       clipboard_text = g_malloc ((length + 1) * sizeof (gchar));
       edit_cpy_sel_mem (tmaid, &tmaid->select, &tmaid->cursor, clipboard_text);
@@ -610,24 +640,23 @@ command_paste (gpointer   callback_data,
 {
   /* ja:Windowsのセレクションの実装があやしいので暫定的処理 */
 #ifdef G_OS_WIN32
-  gchar *text = NULL;
-  TmaidHistory *d;
-  TmaidWindow *tmaid;
-  GdkPoint select;
-  HGLOBAL hGlobal;
-  LPSTR lpszTextA;
-  LPWSTR lpszTextW;
-
   if (OpenClipboard (GDK_WINDOW_HWND (window->window)))
     {
+      gchar *text = NULL;
+      HGLOBAL hGlobal;
+
       if (hGlobal = GetClipboardData (CF_UNICODETEXT))
         {
+          LPWSTR lpszTextW;
+
           if (lpszTextW = GlobalLock (hGlobal))
             text = g_utf16_to_utf8 (lpszTextW, -1, NULL, NULL, NULL);
           GlobalUnlock (hGlobal);
         }
       if (!text && (hGlobal = GetClipboardData (CF_TEXT)))
         {
+          LPSTR lpszTextA;
+
           if (lpszTextA = GlobalLock (hGlobal))
             text = g_locale_to_utf8 (lpszTextA, -1, NULL, NULL, NULL);
           GlobalUnlock (hGlobal);
@@ -635,18 +664,21 @@ command_paste (gpointer   callback_data,
       CloseClipboard ();
       if (text)
         {
-          tmaid = g_object_get_data(G_OBJECT
-                    (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
+          gint page_num;
+          GdkPoint select;
+          TmaidHistory *d;
+          TmaidWindow *tmaid;
+
+          page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
+          tmaid = misc_mdi_get_data (MISC_MDI (mdi), page_num);
           select = tmaid->select;
           d = edit_operation (tmaid, text, g_strlen (text), TRUE, FALSE);
+          g_free (text);
           d->next = tmaid->undo;
           tmaid->undo = d;
           if (delete_list (&tmaid->redo) > 0 || !d->next || select.x >= 0)
             set_menu_bar (tmaid);
-          tmaid_change_edit (tmaid, TRUE);
-          g_free (text);
+          misc_mdi_set_edited (MISC_MDI (mdi), page_num, TRUE);
         }
     }
 #else /* not G_OS_WIN32 */
@@ -661,18 +693,17 @@ command_delete (gpointer   callback_data,
                 guint      callback_action,
                 GtkWidget *widget)
 {
+  gint page_num;
   TmaidHistory *d;
   TmaidWindow *tmaid;
 
-  tmaid = g_object_get_data (G_OBJECT
-                (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
+  page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
+  tmaid = misc_mdi_get_data (MISC_MDI (mdi), page_num);
   d = edit_operation (tmaid, NULL, 0, TRUE, FALSE);
   d->next = tmaid->undo;
   tmaid->undo = d;
   set_menu_bar (tmaid);
-  tmaid_change_edit (tmaid, TRUE);
+  misc_mdi_set_edited (MISC_MDI (mdi), page_num, TRUE);
 }
 
 
@@ -681,17 +712,9 @@ command_margin (gpointer   callback_data,
                 guint      callback_action,
                 GtkWidget *widget)
 {
-  TmaidWindow *tmaid;
-
   if (misc_message_box ("Text maid", _("Return at Right Margin?"), 1,
                                             _("_Yes"), _("_No"), NULL) == 0)
-    {
-      tmaid = g_object_get_data (G_OBJECT
-                (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
-      margin_operation (tmaid);
-    }
+    margin_operation ((TmaidWindow *)misc_mdi_get_data (MISC_MDI (mdi), -1));
 }
 
 
@@ -700,17 +723,9 @@ command_tab (gpointer   callback_data,
              guint      callback_action,
              GtkWidget *widget)
 {
-  TmaidWindow *tmaid;
-
   if (misc_message_box ("Text maid", _("Convert Tab to Space?"), 1,
                                             _("_Yes"), _("_No"), NULL) == 0)
-    {
-      tmaid = g_object_get_data (G_OBJECT
-                (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
-      tab_operation (tmaid);
-    }
+    tab_operation ((TmaidWindow *)misc_mdi_get_data (MISC_MDI (mdi), -1));
 }
 
 
@@ -720,23 +735,23 @@ command_valchr (gpointer   callback_data,
                 GtkWidget *widget)
 {
   gchar valchr;
-  TmaidHistory *d;
-  TmaidWindow *tmaid;
-  GdkPoint select;
 
   if (valchr_dialog (&valchr))
     {
-      tmaid = g_object_get_data (G_OBJECT
-                (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
+      gint page_num;
+      GdkPoint select;
+      TmaidHistory *d;
+      TmaidWindow *tmaid;
+
+      page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
+      tmaid = misc_mdi_get_data (MISC_MDI (mdi), page_num);
       select = tmaid->select;
       d = edit_operation (tmaid, &valchr, 1, TRUE, FALSE);
       d->next = tmaid->undo;
       tmaid->undo = d;
-      if (delete_list(&tmaid->redo) > 0 || !d->next || select.x >= 0)
+      if (delete_list (&tmaid->redo) > 0 || !d->next || select.x >= 0)
         set_menu_bar (tmaid);
-      tmaid_change_edit (tmaid, TRUE);
+      misc_mdi_set_edited (MISC_MDI (mdi), page_num, TRUE);
     }
 }
 
@@ -749,14 +764,11 @@ command_jump (gpointer   callback_data,
   JumpDialog jmp;
   TmaidWindow *tmaid;
 
-  tmaid = gtk_object_get_data (GTK_OBJECT
-                (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
+  tmaid = misc_mdi_get_data (MISC_MDI (mdi), -1);
   jmp.max = tmaid->max;
   jmp.cursor = tmaid->cursor.y + 1;
   if (jump_dialog (&jmp) && tmaid->cursor.y != jmp.cursor - 1)
-    jump_operation (tmaid,jmp.cursor);
+    jump_operation (tmaid, jmp.cursor);
 }
 
 
@@ -769,10 +781,7 @@ command_all (gpointer   callback_data,
   GdkPoint top;
   TmaidWindow *tmaid;
 
-  tmaid = g_object_get_data (G_OBJECT
-                (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
+  tmaid = misc_mdi_get_data (MISC_MDI (mdi), -1);
   if (tmaid->max <= 1 && tmaid->start->length <= 0)
     return;
   top = tmaid->top;
@@ -808,15 +817,10 @@ command_find (gpointer   callback_data,
               guint      callback_action,
               GtkWidget *widget)
 {
-  gchar *text;
-  gint i;
   FindString fd;
   TmaidWindow *tmaid;
 
-  tmaid = g_object_get_data (G_OBJECT
-                (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
+  tmaid = misc_mdi_get_data (MISC_MDI (mdi), -1);
   fd.text = NULL;
   fd.arrow = find_arrow;
   fd.ignorecase = find_ignorecase;
@@ -826,11 +830,13 @@ command_find (gpointer   callback_data,
                                         && tmaid->cursor.x < tmaid->select.x);
   if (find_dialog (&fd))
     {
+      gint i;
+
       set_menu_bar (tmaid);
       find_arrow = fd.arrow;
       find_ignorecase = fd.ignorecase;
       for (i = 0; i < find_num; i++)
-        if (!g_strcmp (fd.text, find_text[i]))
+        if (g_strcmp (fd.text, find_text[i]) == 0)
           break;
       if (i == find_num)
         {
@@ -847,6 +853,8 @@ command_find (gpointer   callback_data,
           g_free (fd.text);
           if (i > 0)
             {
+              gchar *text;
+
               text = find_text[i];
               g_memmove (find_text + 1, find_text, i * sizeof (gchar *));
               find_text[0] = text;
@@ -862,14 +870,10 @@ command_next (gpointer   callback_data,
               guint      callback_action,
               GtkWidget *widget)
 {
-  gchar *text;
   GtkWidget *dialog;
   TmaidWindow *tmaid;
 
-  tmaid = g_object_get_data (G_OBJECT
-                (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
+  tmaid = misc_mdi_get_data (MISC_MDI (mdi), -1);
   /* en:Dialog Box */
   /* ja:ダイアログボックス */
   userbreak = TRUE;
@@ -881,9 +885,11 @@ command_next (gpointer   callback_data,
   gtk_widget_destroy (dialog);
   if (userbreak && tmaid->select.x < 0)
     {
+      gchar *text;
+
       text = g_strdup_printf (_("'%s' was not fount."), find_text[0]);
       misc_message_box ("Text maid", text, 0, _("OK"), NULL);
-      g_free(text);
+      g_free (text);
       set_menu_bar (tmaid);
     }
 }
@@ -895,19 +901,12 @@ command_replace (gpointer   callback_data,
                  GtkWidget *widget)
 {
   gboolean replace = FALSE;
-  gchar *text;
-  gint i, bytes, count = 0, data_pos, result;
-  GdkPoint select;
-  GtkWidget *dialog = NULL;
-  TmaidHistory *d;
+  gint page_num, result;
   TmaidWindow *tmaid;
-  ReplaceInfo ri;
   ReplaceString rp;
 
-  tmaid = g_object_get_data (G_OBJECT
-                (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
+  page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
+  tmaid = misc_mdi_get_data (MISC_MDI (mdi), page_num);
   rp.src = rp.dst = NULL;
   rp.arrow = replace_arrow;
   rp.ignorecase = replace_ignorecase;
@@ -918,11 +917,17 @@ command_replace (gpointer   callback_data,
   result = replace_dialog (&rp);
   if (result != REPLACE_CANCEL)
     {
+      gchar *text;
+      gint i, count = 0;
+      GdkPoint select;
+      GtkWidget *dialog = NULL;
+      ReplaceInfo ri;
+
       set_menu_bar (tmaid);
       replace_arrow = rp.arrow;
       replace_ignorecase = rp.ignorecase;
       for (i = 0; i < find_num; i++)
-        if (!g_strcmp(rp.src,find_text[i]))
+        if (g_strcmp (rp.src, find_text[i]) == 0)
           break;
       if (i == find_num)
         {
@@ -944,7 +949,7 @@ command_replace (gpointer   callback_data,
       if (rp.dst && rp.dst[0] != '\0')
         {
           for (i = 0; i < replace_num; i++)
-            if (!g_strcmp (rp.dst, replace_text[i]))
+            if (g_strcmp (rp.dst, replace_text[i]) == 0)
               break;
           if (i == replace_num)
             {
@@ -976,6 +981,9 @@ command_replace (gpointer   callback_data,
       userbreak = TRUE;
       do
         {
+          gint bytes, data_pos;
+          TmaidHistory *d;
+
           /* en:Dialog Box */
           /* ja:ダイアログボックス */
           if (!dialog)
@@ -1024,7 +1032,7 @@ command_replace (gpointer   callback_data,
           tmaid->undo = d;
           if (delete_list (&tmaid->redo) > 0 || !d->next)
             set_menu_bar (tmaid);
-          tmaid_change_edit (tmaid, TRUE);
+          misc_mdi_set_edited (MISC_MDI (mdi), page_num, TRUE);
         }
       while (result != REPLACE_CANCEL);
       /* en:Dialog Box */
@@ -1074,7 +1082,7 @@ command_conf_ok (GtkWidget *widget,
                                                 g_list_nth_data (glist, i));
   g_list_free (glist);
   /* ja:ノートの位置 */
-  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), n_pos);
+  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (mdi), n_pos);
 }
 
 
