@@ -18,15 +18,16 @@
 */
 #include "abort.h"
 #include "charset.h"
-#include "charlist.h"
 #include "file.h"
 #include "general.h"
 #include "prop.h"
 #include "sigfile.h"
 #include "sigkey.h"
 #include "sigmain.h"
+#include "misc/charuty.h"
 #include "misc/fileio.h"
 #include "misc/misc.h"
+#include "misc/misc_mdi.h"
 
 
 /******************************************************************************
@@ -85,155 +86,17 @@ file_set_history (const gchar *file)
 }
 
 
-/*  ja:ファイルを加える
-    file,加えようとするファイルのフルパス
-    same,フルパスに付属する数値
-     RET,加えようとするファイルのラベル                                     */
-gchar *
-file_add_edit (const gchar *file, gint *same)
-{
-  gchar *label, *utf8str;
-  const gchar *name0, *name1;
-  gint i;
-  GList *glist;
-  GtkWidget *child;
-  TmaidWindow *tmaid;
-
-  *same=-1;
-  /* ja:ファイル名を調べる */
-  glist = gtk_container_children (GTK_CONTAINER (notebook));
-  /* ja:同名同パスを調べる */
-  for (i = g_list_length (glist) - 1; i >= 0; i--)
-    {
-      child = g_list_nth_data (glist, i);
-      tmaid = g_object_get_data (G_OBJECT (child), "user_data");
-      if (!g_strfilecmp (file, tmaid->file))
-        {
-          if (tmaid->same < 0)
-            {
-              tmaid->same = 0;
-              utf8str = g_filename_to_utf8 (tmaid->file, -1, NULL, NULL, NULL);
-              label = g_strdup_printf ("%s:%d", utf8str, tmaid->same);
-              g_free (utf8str);
-              gtk_label_set_text (GTK_LABEL (tmaid->label), label);
-              g_free (label);
-            }
-            if (*same <= tmaid->same)
-              *same = tmaid->same + 1;
-        }
-    }
-  if (*same >= 0)
-    {
-      /* ja:同名同パスがあるとき */
-      utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
-      label = g_strdup_printf ("%s:%d", utf8str, *same);
-      g_free (utf8str);
-    }
-  else
-    {
-      /* ja:同名同パスはないとき */
-      /* ja:同名異パスを調べる */
-      name0 = g_basename (file);
-      for (i = g_list_length (glist) - 1; i >= 0; i--)
-        {
-          child = g_list_nth_data (glist, i);
-          tmaid = g_object_get_data (G_OBJECT (child), "user_data");
-          name1 = g_basename (tmaid->file);
-          if (!g_strfilecmp (name0, name1))
-            {
-              if (tmaid->same < 0)
-                {
-                  utf8str = g_filename_to_utf8 (tmaid->file, -1,
-                                                            NULL, NULL, NULL);
-                  gtk_label_set_text (GTK_LABEL (tmaid->label), utf8str);
-                  g_free (utf8str);
-                }
-              break;
-            }
-        }
-      label = g_filename_to_utf8 (i >= 0 ? file : name0, -1, NULL, NULL, NULL);
-    }
-  g_list_free (glist);
-  return label;
-}
-
-
-/*  ja:ファイルを削除する
-    file,削除しようとするファイルのフルパス                                 */
-void
-file_delete_edit (const gchar *file)
-{
-  gchar *utf8str;
-  const gchar *name0,*name1;
-  gint i,count=0;
-  GList *glist;
-  TmaidWindow *tmaid;
-  GtkWidget *child;
-
-  /* ja:ファイル名を調べる */
-  glist = gtk_container_children (GTK_CONTAINER (notebook));
-  /* ja:同名異パスを調べる */
-  name0 = g_basename (file);
-  for (i = g_list_length (glist) - 1; i >= 0; i--)
-    {
-      tmaid = g_object_get_data (g_list_nth_data (glist, i), "user_data");
-      name1 = g_basename (tmaid->file);
-      if (g_strfilecmp (name0, name1) == 0)
-        count++;
-    }
-  if (count <= 2)
-    {
-      /* ja:同名異パスのファイルを通常表示に変更する */
-      for (i = g_list_length (glist) - 1; i >= 0; i--)
-        {
-          child = g_list_nth_data (glist, i);
-          tmaid = g_object_get_data (G_OBJECT (child), "user_data");
-          name1 = g_basename (tmaid->file);
-          if (!g_strfilecmp (name0, name1))
-            {
-              utf8str = g_filename_to_utf8 (name1, -1, NULL, NULL, NULL);
-              gtk_label_set_text (GTK_LABEL (tmaid->label), utf8str);
-              g_free (utf8str);
-            }
-        }
-    }
-  else
-    {
-      /* ja:同名同パスを調べる */
-      count = 0;
-      for (i = g_list_length (glist) - 1; i >= 0; i--)
-        {
-          tmaid = g_object_get_data (g_list_nth_data (glist, i), "user_data");
-          if (!g_strfilecmp (file, tmaid->file))
-            count++;
-        }
-      if (count <= 2)/* ja:同名同パスのファイルをフルパス表示に変更する */
-        for (i = g_list_length (glist) - 1; i >= 0; i--)
-          {
-            child = g_list_nth_data (glist, i);
-            tmaid = g_object_get_data (G_OBJECT (child), "user_data");
-            if (!g_strfilecmp (file, tmaid->file))
-              {
-                tmaid->same = -1;
-                utf8str = g_filename_to_utf8 (tmaid->file, -1,
-                                                            NULL, NULL, NULL);
-                gtk_label_set_text (GTK_LABEL (tmaid->label), utf8str);
-                g_free (utf8str);
-              }
-          }
-    }
-}
-
-
 /******************************************************************************
 *                                                                             *
 * ja:ファイル入力関数群                                                       *
 *                                                                             *
 ******************************************************************************/
 /*  ja:TXTファイルを開く
-    tmaid,ウインドウ情報                                                    */
+     file,ファイル名
+    tmaid,ウインドウ情報                                                 */
 void
-file_open_text (TmaidWindow *tmaid)
+file_open_text (const gchar *file,
+                TmaidWindow *tmaid)
 {
   gchar buf[16384], **charset, *ptr, *inbuf = NULL, *outbuf = NULL;
   gssize bytes_read, bytes_written, leng, length = 0;
@@ -255,7 +118,7 @@ file_open_text (TmaidWindow *tmaid)
   dialog = abort_dialog (_("Reading"));
   gtk_grab_add (dialog);
   /* ja:読み込み */
-  fio = fileio_open (tmaid->file, FILEIO_ACCESS_READ, FILEIO_SHARE_READ,
+  fio = fileio_open (file, FILEIO_ACCESS_READ, FILEIO_SHARE_READ,
                                                     FILEIO_MODE_OPEN_EXISTING);
   if (fio)
     {
@@ -302,7 +165,7 @@ file_open_text (TmaidWindow *tmaid)
           g_free (tmp);
           for (j = 0; j < ftnum; j++)
             {
-              tmp = charlist_remove (ftype[j].charset, charset[i]);
+              tmp = charuty_remove (ftype[j].charset, charset[i]);
               g_free (ftype[j].charset);
               ftype[j].charset = tmp ? tmp : g_strdup ("UTF-8");
             }
@@ -462,8 +325,7 @@ file_open_text (TmaidWindow *tmaid)
                     else
                       break;
                 }
-              if (i < j && j < p->length && p->text[j++]
-                                                    == G_SEARCHPATH_SEPARATOR)
+              if (i < j && j < p->length && p->text[j++] == ';')
                 {
                   i -= 2;
                   leng = g_unichar_to_utf8 (c, buf);
@@ -516,38 +378,25 @@ file_open_text (TmaidWindow *tmaid)
 TmaidWindow *
 file_open_edit (const gchar *file)
 {
-  gchar *ext, *p, *text, **charset, *utf8str;
-  gint i;
+  gchar *filename;
+  gint i, page_num;
   TmaidWindow *tmaid;
-  GdkBitmap *mask;
-  GdkPixmap *pixmap;
   GdkRectangle rc;
-  GtkWidget *table, *menu_shell, *hbox, *button, *frame;
+  GtkWidget *table;
 #ifndef G_OS_WIN32
   gint x, y;
   GtkWidget *preedit;
 #endif /* not G_OS_WIN32 */
-  static gint fcount = 0;   /* ja:新規ファイルの数 */
-  static gchar *xpm[] = {
-"8 7 2 1",
-"   c None",
-".  c #000000",
-"..    ..",
-" ..  .. ",
-"  ....  ",
-"   ..   ",
-"  ....  ",
-" ..  .. ",
-"..    .."};
 
   tmaid = g_malloc0 (sizeof (TmaidWindow));
-  tmaid->same = -1;
   tmaid->select.x = -1;
   if (!file)
     {
+      gchar **charset;
+      static gint fcount = 0;   /* ja:新規ファイルの数 */
+
       /* ja:新規 */
-      tmaid->file = g_strdup_printf ("new%04d", fcount++ % 1000);
-      tmaid->create = TRUE;
+      filename = g_strdup_printf ("new%04d", fcount++ % 1000);
       property_copy (&tmaid->ft, ftype);
       if (!ftype[0].associate)
         tmaid->ft.ft_id = -1;
@@ -565,10 +414,16 @@ file_open_edit (const gchar *file)
     }
   else
     {
-      tmaid->file = fileio_get_full_path (file);
-      text = g_filename_to_utf8 (g_basename (file), -1, NULL, NULL, NULL);
+      gchar *name, *text;
+
+      filename = fileio_get_full_path (file);
+      name = g_path_get_basename (file);
+      text = g_filename_to_utf8 (name, -1, NULL, NULL, NULL);
+      g_free (name);
       for (i = 0; i < ftnum; i++)
         {
+          gchar *ext, *p;
+
           ext = g_strdup (ftype[i].ext);
           for (p = g_strtok (ext, G_SEARCHPATH_SEPARATOR_S); p;
                                 p = g_strtok (NULL, G_SEARCHPATH_SEPARATOR_S))
@@ -580,18 +435,14 @@ file_open_edit (const gchar *file)
       g_free (text);
       if (i >= ftnum)
       i = 0;
-      tmaid->create = FALSE;
       property_copy (&tmaid->ft, ftype + i);
       if (!ftype[i].associate)
         tmaid->ft.ft_id = -1;
       tmaid->ft.crlf = ftype[i].crlf;
       tmaid->ft.charset = g_strdup (ftype[i].charset);
-      file_open_text (tmaid);
+      file_open_text (file, tmaid);
     }
 
-  /* ja:ファイル名を調べる */
-  text = file_add_edit (tmaid->file, &tmaid->same);
-
   /* ja:Input Method */
   tmaid->im_context = gtk_im_multicontext_new ();
   g_signal_connect (tmaid->im_context, "commit",
@@ -684,41 +535,11 @@ file_open_edit (const gchar *file)
   gtk_table_attach (GTK_TABLE (table), tmaid->vscroll, 1, 2, 0, 1,
                             GTK_FILL,GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
   g_object_set_data (G_OBJECT(table), "user_data", tmaid);
-  /* ja:メニュー */
-  menu_shell = gtk_item_factory_get_widget (ifactory_menu, "<main>/Window");
-  utf8str = g_filename_to_utf8 (tmaid->file, -1, NULL, NULL, NULL);
-  g_free (utf8str);
-  tmaid->menu_item = gtk_menu_item_new_with_label (tmaid->file);
-  g_signal_connect (G_OBJECT (tmaid->menu_item), "activate",
-                            G_CALLBACK (signal_activate_menu_window), table);
-  gtk_menu_append (GTK_MENU (menu_shell), tmaid->menu_item);
-  gtk_widget_show (tmaid->menu_item);
-  /* ja:ボタン */
-  button = gtk_button_new ();
-  frame = gtk_frame_new (NULL);
-  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask, NULL, xpm);
-  gtk_container_add (GTK_CONTAINER (frame), gtk_pixmap_new (pixmap, mask));
-  gtk_container_add (GTK_CONTAINER (button), frame);
-  g_signal_connect (G_OBJECT (button), "clicked",
-                                        G_CALLBACK (signal_clicked), table);
-  /* ja:ラベル */
-  tmaid->label = gtk_label_new (text);
-  g_free (text);
-  /* ja:マーク */
-  tmaid->mark = gtk_label_new ("*");
   /* ja:表示 */
-  hbox = gtk_hbox_new (FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), tmaid->label, TRUE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), tmaid->mark, FALSE, FALSE, 0);
-  gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), table, hbox);
-  gtk_widget_show (hbox);
-  gtk_widget_show (tmaid->label);
-  gtk_widget_show_all (button);
+  page_num = misc_mdi_append_page (MISC_MDI (mdi),
+                                                table, filename, !file, tmaid);
   gtk_widget_show_all (table);
-  gtk_notebook_set_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_page_num (GTK_NOTEBOOK (notebook), table));
+  gtk_notebook_set_page (GTK_NOTEBOOK (mdi), page_num);
   gtk_widget_grab_focus (tmaid->drawing);
 
   /* ja:Input Method */
@@ -732,8 +553,9 @@ file_open_edit (const gchar *file)
   rc.height = tmaid->font_height;
   gtk_im_context_set_cursor_location (tmaid->im_context, &rc);
   /* ja:新規作成ではなく、ファイルが存在する時には履歴に加える */
-  if (file && history > 0 && fileio_isfile (tmaid->file))
-    file_set_history(tmaid->file);
+  if (file && history > 0 && fileio_isfile (file))
+    file_set_history (filename);
+  g_free (filename);
   /* ja:右マージン */
   if (tmaid->ft.limit)
     modify_margin (tmaid);
