@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -39,24 +39,12 @@ void
 file_set_history (const gchar *file);
 
 
-/*  ja:ファイルを加える
-    file,加えようとするファイルのフルパス
-     RET,加えようとするファイルのラベル                                     */
-gchar *
-file_add_edit (const gchar *file,
-               gint        *same);
-
-
-/*  ja:ファイルを削除する
-    file,削除しようとするファイルのフルパス                                 */
-void
-file_delete_edit (const gchar *file);
-
-
 /*  ja:TXTファイルを開く
+     file,ファイル名
     tmaid,ウインドウ情報                                                 */
 void
-file_open_text (TmaidWindow *tmaid);
+file_open_text (const gchar *file,
+                TmaidWindow *tmaid);
 
 
 /*  ja:ファイルを開く
