@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,10 +17,10 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 #include <gdk/gdkkeysyms.h>
-#include "charlist.h"
 #include "format.h"
 #include "general.h"
 #include "prop.h"
+#include "misc/misc_mdi.h"
 
 
 /******************************************************************************
@@ -161,26 +161,24 @@ static void
 format_dialog_clicked_edit (GtkWidget *widget,
                             gpointer   user_data)
 {
-  gint i;
-  GList *glist;
-  GtkTreeIter iter;
-  TmaidWindow *tmaid;
 
   if (property_dialog (ftype + ftpos))
     {
+      gint i;
+      GtkTreeIter iter;
+
       if (gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (store),
                                                         &iter, NULL, ftpos))
         gtk_list_store_set (store, &iter, 0, ftype[ftpos].text, -1);
       /* ja:同一ファイルタイプをすべて変更 */
-      glist = gtk_container_children (GTK_CONTAINER (notebook));
-      for (i = g_list_length (glist) - 1; i >= 0; i--)
+      for (i = misc_mdi_get_n_pages (MISC_MDI (mdi)) - 1; i >= 0; i--)
         {
-          tmaid = g_object_get_data (G_OBJECT (g_list_nth_data (glist, i)),
-                                                                "user_data");
+          TmaidWindow *tmaid;
+
+          tmaid = misc_mdi_get_data (MISC_MDI (mdi), i);
           if (tmaid->ft.ft_id == ftype[ftpos].ft_id)
             property_change (tmaid, ftype + ftpos);
         }
-      g_list_free (glist);
       charlist_renewal_all (ftype[ftpos].charset, -1);
     }
 }
@@ -204,7 +202,6 @@ format_dialog (void)
 {
   gint i;
   GtkCellRenderer *renderer;
-  GtkTreeIter iter;
   GtkTreeSelection *select;
   GtkTreeViewColumn *column;
   GtkWidget *dialog, *button5, *tview, *scroll, *hbox, *vbox;
@@ -230,6 +227,8 @@ format_dialog (void)
                                     G_CALLBACK (format_dialog_changed), NULL);
   for (i = 0; i < ftnum; i++)
     {
+      GtkTreeIter iter;
+
       gtk_list_store_append (store, &iter);
       gtk_list_store_set (store, &iter, 0, ftype[i].text, -1);
     }
@@ -244,7 +243,7 @@ format_dialog (void)
   button2 = gtk_button_new_with_mnemonic (_("_Up"));
   button3 = gtk_button_new_with_mnemonic (_("_Down"));
   button4 = gtk_button_new_with_mnemonic (_("_Edit"));
-  button5=gtk_button_new_with_label(_("Close"));
+  button5 = gtk_button_new_with_label (_("Close"));
   g_signal_connect (G_OBJECT (button0), "clicked",
                         G_CALLBACK (format_dialog_clicked_new), NULL);
   g_signal_connect (G_OBJECT (button1), "clicked",
