@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "print.h"
 #include "misc/misc.h"
 #ifdef G_OS_WIN32
+# include "misc/misc_mdi.h"
 # include <gdk/gdkwin32.h>
 #endif /* G_OS_WIN32 */
 
@@ -113,7 +114,9 @@ print_out (TmaidWindow *tmaid,
     }
   g_memset (&di, 0, sizeof (DOCINFO));
   di.cbSize = sizeof (DOCINFO);
-  utf8str = g_filename_to_utf8 (tmaid->file, -1, NULL, NULL, NULL);
+  utf8str = g_filename_to_utf8 (misc_mdi_get_file (MISC_MDI (mdi),
+                        misc_mdi_get_page_from_data (MISC_MDI (mdi), tmaid)),
+                                                        -1, NULL, NULL, NULL);
 # ifdef UNICODE
   name = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
 # else /* not UNICODE */
