@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 #include <gdk/gdkkeysyms.h>
-#include "charlist.h"
 #include "charset.h"
 #include "edit.h"
 #include "ftype.h"
@@ -25,6 +24,8 @@
 #include "prop.h"
 #include "sigfile.h"
 #include "misc/misc.h"
+#include "misc/misc_charlist.h"
+#include "misc/misc_charset.h"
 
 
 /******************************************************************************
@@ -139,12 +140,12 @@ property_change (TmaidWindow *tmaid,
 *                                                                             *
 ******************************************************************************/
 static gboolean property_clicked_ok, color_list[12];
-static gchar *charset, *font_name;
+static gchar *charset_list, *font_name;
 static gint ft_id;
 static GdkColor gdk_color[12];
 static GtkListStore *store;
 static GtkWidget *dialog;
-static GtkWidget *button0, *button1, *button4, *combo0, *combo1;
+static GtkWidget *button0, *button1, *button4, *combo, *charset;
 static GtkWidget *check0, *check1, *check2, *check3, *check4;
 static GtkWidget *check5, *check6, *check7, *check8, *check9;
 static GtkWidget *check10, *check11, *check12, *check13, *check14, *check15;
@@ -158,12 +159,10 @@ property_dialog_ok_sensitive (GtkWidget *widget,
                               gpointer   user_data)
 {
   gtk_widget_set_sensitive (button4,
-                (!entry0
-                    || g_strlen (gtk_entry_get_text (GTK_ENTRY (entry0))) > 0)
-                && (!entry1
+            (!entry0 || g_strlen (gtk_entry_get_text (GTK_ENTRY (entry0))) > 0)
+            && (!entry1
                     || g_strlen (gtk_entry_get_text (GTK_ENTRY (entry1))) > 0)
-                && (!combo1 || charlist_is_valid (gtk_entry_get_text
-                                    (GTK_ENTRY (GTK_COMBO (combo1)->entry)))));
+            && (!charset || misc_charset_is_valid (MISC_CHARSET (charset))));
 }
 
 
@@ -188,8 +187,8 @@ property_dialog_toggled_radio (GtkWidget *widget,
 {
   gboolean result;
 
-  result = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio3))
-                    || gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio4));
+  result = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio3))
+                || gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio4));
   gtk_widget_set_sensitive (radio5, result);
   gtk_widget_set_sensitive (radio6, result);
   property_dialog_ok_sensitive (widget, user_data);
@@ -288,20 +287,42 @@ property_dialog_clicked_color (GtkWidget *widget,
 }
 
 
-/* ja:キャラクターセットの設定ボタンが押された */
+/* ja:文字符号化方式の設定でOKが押されたとき */
 static void
-property_dialog_clicked_charset (GtkWidget *widget,
-                                 gpointer   user_data)
+property_dialog_clicked_charlist_ok (GtkWidget *widget,
+                                     GtkWidget *dialog_charlist)
 {
-  gchar *result;
+  g_free (charset_list);
+  charset_list = misc_charlist_get_charlist (MISC_CHARLIST (dialog_charlist));
+  property_dialog_ok_sensitive (NULL, NULL);
+  gtk_widget_destroy (dialog_charlist);
+}
 
-  result = charlist_dialog (charset);
-  if (result)
-    {
-      g_free (charset);
-      charset = result;
-      property_dialog_ok_sensitive (NULL, NULL);
-    }
+
+/* ja:文字符号化方式の設定ボタンが押された */
+static void
+property_dialog_clicked_charlist (GtkWidget *widget,
+                                  gpointer   user_data)
+{
+  gint i;
+  GtkWidget *dialog_charlist;
+
+  dialog_charlist = misc_charlist_new ();
+  g_signal_connect (G_OBJECT (dialog_charlist), "destroy",
+                                            G_CALLBACK (gtk_main_quit), NULL);
+  g_signal_connect (G_OBJECT (MISC_CHARLIST (dialog_charlist)->ok_button),
+                "clicked", G_CALLBACK (property_dialog_clicked_charlist_ok),
+                                                            dialog_charlist);
+  g_signal_connect_swapped
+    (G_OBJECT (MISC_CHARLIST (dialog_charlist)->cancel_button),
+    "clicked", G_CALLBACK (gtk_widget_destroy), GTK_OBJECT (dialog_charlist));
+  misc_charlist_set_charlist (MISC_CHARLIST (dialog_charlist), charset_list);
+  for (i = 0; i < ftnum; i++)
+    misc_charlist_add_candidate (MISC_CHARLIST (dialog_charlist),
+                                                            ftype[i].charset);
+  gtk_widget_show (dialog_charlist);
+  gtk_grab_add (dialog_charlist);
+  gtk_main ();
 }
 
 
@@ -404,10 +425,10 @@ property_dialog_clicked_ftype (GtkWidget *widget,
           gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ftype[pos].hexcase
                                                     ? radio5 : radio6), TRUE);
           if (ftype[pos].crlf < 3)
-            gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo0)->entry),
+            gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry),
                     gtk_label_get_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN
                     (g_list_nth_data (gtk_container_get_children
-                        (GTK_CONTAINER (GTK_COMBO (combo0)->list)),
+                        (GTK_CONTAINER (GTK_COMBO (combo)->list)),
                                                 ftype[pos].crlf))))));
           gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin0),
                                                             ftype[pos].margin);
@@ -475,14 +496,13 @@ property_dialog_clicked_ok (GtkWidget *widget,
   else
     ft->writemode = 0;
   ft->hexcase = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio5));
-  if (combo0)
+  if (combo)
     {
       const gchar *text;
       GList *glist;
 
-      text = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo0)->entry));
-      glist = gtk_container_children
-                                    (GTK_CONTAINER (GTK_COMBO (combo0)->list));
+      text = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo)->entry));
+      glist = gtk_container_children (GTK_CONTAINER (GTK_COMBO (combo)->list));
       for (ft->crlf = g_list_length (glist) - 1; ft->crlf >= 0; ft->crlf--)
         if (!g_strcmp (gtk_label_get_text (GTK_LABEL (gtk_bin_get_child
                         (GTK_BIN (g_list_nth_data (glist, ft->crlf))))), text))
@@ -492,15 +512,12 @@ property_dialog_clicked_ok (GtkWidget *widget,
         ft->crlf = 0;
     }
   g_free (ft->charset);
-  if (combo1)
-    {
-      ft->charset = g_strdup (gtk_entry_get_text
-                                    (GTK_ENTRY (GTK_COMBO (combo1)->entry)));
-    }
+  if (charset)
+    ft->charset = misc_charset_get_charset (MISC_CHARSET (charset));
   if (button1)
     {
-      ft->charset = charset;
-      charset = NULL;
+      ft->charset = charset_list;
+      charset_list = NULL;
     }
   g_memmove (ft->color, gdk_color, sizeof (GdkColor) * 12);
   g_free (ft->font_name);
@@ -539,8 +556,8 @@ property_dialog_key_press (GtkWidget   *widget,
 gboolean
 property_dialog (FileType *ft)
 {
-  gchar **name, *xpm[18];
-  gint i, j;
+  gchar *xpm[18];
+  gint i;
   GList *glist;
   GtkCellRenderer *renderer;
   GtkTreeIter iter;
@@ -585,7 +602,7 @@ property_dialog (FileType *ft)
     {
       button1 = gtk_button_new_with_label (_("Auto Negotiation"));
       g_signal_connect (G_OBJECT (button1), "clicked",
-                        G_CALLBACK (property_dialog_clicked_charset), NULL);
+                        G_CALLBACK (property_dialog_clicked_charlist), NULL);
       GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);
     }
   else
@@ -683,45 +700,30 @@ property_dialog (FileType *ft)
   glist = g_list_append (glist, "LF");
   if (ft->text && ft->ext)
     glist = g_list_append (glist, _("Auto"));
-  combo0 = gtk_combo_new();
-  gtk_combo_set_popdown_strings (GTK_COMBO (combo0), glist);
-  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo0)->entry),
+  combo = gtk_combo_new();
+  gtk_combo_set_popdown_strings (GTK_COMBO (combo), glist);
+  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry),
                                             g_list_nth_data (glist, ft->crlf));
-  gtk_editable_set_editable (GTK_EDITABLE (GTK_COMBO (combo0)->entry), FALSE);
-  g_signal_connect (G_OBJECT (GTK_COMBO (combo0)->entry), "activate",
+  gtk_editable_set_editable (GTK_EDITABLE (GTK_COMBO (combo)->entry), FALSE);
+  g_signal_connect (G_OBJECT (GTK_COMBO (combo)->entry), "activate",
                                     G_CALLBACK (property_dialog_activate), ft);
-  g_signal_connect (G_OBJECT (GTK_COMBO (combo0)->entry),"changed",
+  g_signal_connect (G_OBJECT (GTK_COMBO (combo)->entry),"changed",
                             G_CALLBACK (property_dialog_ok_sensitive), NULL);
+  /* ja:文字符号化方式 */
   if (!ft->text && !ft->ext)
     {
-      glist = NULL;
+      charset = misc_charset_new ();
       for (i = 0; i < ftnum; i++)
-        {
-          name = g_strsplit (ftype[i].charset, ",", G_MAXINT);
-          for (j = 0; name[j] != NULL; j++)
-            if (!g_list_find_custom (glist, name[j],
-                                                (GCompareFunc)g_strcasecmp))
-              glist = g_list_insert_sorted (glist, g_strdup (name[j]),
-                                                (GCompareFunc)g_strcasecmp);
-          g_strfreev (name);
-        }
-      combo1 = gtk_combo_new();
-      gtk_combo_set_popdown_strings (GTK_COMBO (combo1), glist);
-      gtk_combo_set_value_in_list (GTK_COMBO (combo1), FALSE, FALSE);
-      gtk_combo_set_case_sensitive (GTK_COMBO (combo1), TRUE);
-      gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo1)->entry), ft->charset);
-      gtk_editable_select_region
-                            (GTK_EDITABLE (GTK_COMBO (combo1)->entry), 0, -1);
-      g_signal_connect (G_OBJECT (GTK_COMBO (combo1)->entry), "activate",
+        misc_charset_add_candidate (MISC_CHARSET (charset), ftype[i].charset);
+      misc_charset_set_charset (MISC_CHARSET (charset), ft->charset);
+      g_signal_connect (G_OBJECT (charset), "activate",
                                     G_CALLBACK (property_dialog_activate), ft);
-      g_signal_connect (G_OBJECT (GTK_COMBO (combo1)->entry), "changed",
+      g_signal_connect (G_OBJECT (charset), "changed",
                             G_CALLBACK (property_dialog_ok_sensitive), NULL);
-      for (i = g_list_length (glist) - 1; i >= 0; i--)
-        g_free (g_list_nth_data (glist, i));
     }
   else
     {
-      combo1 = NULL;
+      charset = NULL;
     }
   /* ja:エントリー/ラベル */
   label = NULL;
@@ -749,7 +751,7 @@ property_dialog (FileType *ft)
             if (ft->ft_id == ftype[i].ft_id)
               {
                 label = gtk_label_new (ftype[i].text);
-                  break;
+                break;
               }
           if (!label)
             label = gtk_label_new (_("Unknow File Type"));
@@ -828,7 +830,7 @@ property_dialog (FileType *ft)
                                 G_CALLBACK (property_dialog_changed), NULL);
   xpm[0] = "24 16 1 1";
   for (i = 2; i < 18; i++)
-    xpm[i]="XXXXXXXXXXXXXXXXXXXXXXXX";
+    xpm[i] = "XXXXXXXXXXXXXXXXXXXXXXXX";
   for (i = 0; i < 12; i++)
     {
       xpm[1] = g_strdup_printf ("X\tc #%02X%02X%02X",
@@ -950,18 +952,18 @@ property_dialog (FileType *ft)
   gtk_label_set_text_with_mnemonic (GTK_LABEL (frame_label),
                                                     _("Character _Encoding"));
   gtk_label_set_mnemonic_widget (GTK_LABEL (frame_label),
-                                   GTK_COMBO (combo0)->entry);
+                                                    GTK_COMBO (combo)->entry);
   vbox = gtk_vbox_new (FALSE, SPACING);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);
-  gtk_box_pack_start (GTK_BOX (vbox), combo0, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), combo, FALSE, FALSE, 0);
   if (button1)
     {
       hbox = gtk_hbox_new (FALSE, 0);
       gtk_box_pack_end (GTK_BOX (hbox), button1, FALSE, FALSE, 0);
       gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
     }
-  if (combo1)
-    gtk_box_pack_start (GTK_BOX (vbox), combo1, FALSE, FALSE, 0);
+  if (charset)
+    gtk_box_pack_start (GTK_BOX (vbox), charset, FALSE, FALSE, 0);
   if (check13)
     gtk_box_pack_start (GTK_BOX (vbox), check13, FALSE, FALSE, 0);
   if (check14)
@@ -1026,7 +1028,7 @@ property_dialog (FileType *ft)
 
   ft_id = ft->ft_id;
   g_memmove (gdk_color, ft->color, sizeof (GdkColor) * 12);
-  charset = button1 ? g_strdup (ft->charset) : NULL;
+  charset_list = button1 ? g_strdup (ft->charset) : NULL;
   font_name = g_strdup (ft->font_name);
   property_clicked_ok = FALSE;
 
@@ -1035,7 +1037,7 @@ property_dialog (FileType *ft)
   gtk_widget_show_all (dialog);
   gtk_main ();
 
-  g_free (charset);
+  g_free (charset_list);
   g_free (font_name);
 
   return property_clicked_ok;
