@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 #include <gdk/gdkkeysyms.h>
-#include "charlist.h"
+#include "misc/misc_charset.h"
 #include "reload.h"
 
 
@@ -27,7 +27,7 @@
 *                                                                             *
 ******************************************************************************/
 static gboolean reload_clicked_ok;
-static GtkWidget *dialog, *button0, *check0, *check1, *combo0, *combo1;
+static GtkWidget *dialog, *button0, *check0, *check1, *combo, *charset;
 
 
 /* ja:OKボタンの表示/非表示を判定 */
@@ -36,8 +36,7 @@ reload_dialog_ok_sensitive (GtkWidget *widget,
                             gpointer   user_data)
 {
   gtk_widget_set_sensitive (button0,
-                charlist_is_valid (gtk_entry_get_text
-                                    (GTK_ENTRY (GTK_COMBO (combo1)->entry))));
+                            misc_charset_is_valid (MISC_CHARSET (charset)));
 }
 
 
@@ -46,23 +45,22 @@ static void
 reload_dialog_clicked (GtkWidget *widget,
                        FileType  *ft)
 {
-  const gchar *charset;
+  const gchar *str;
   GList *glist;
 
+  g_free (ft->charset);
   ft->html = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check0));
   ft->esc = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check1));
-  charset = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo0)->entry));
-  glist = gtk_container_children (GTK_CONTAINER (GTK_COMBO (combo0)->list));
+  str = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo)->entry));
+  glist = gtk_container_children (GTK_CONTAINER (GTK_COMBO (combo)->list));
   for (ft->crlf = g_list_length (glist) - 1; ft->crlf >= 0; ft->crlf--)
     if (!g_strcmp (gtk_label_get_text (GTK_LABEL (gtk_bin_get_child
-                    (GTK_BIN (g_list_nth_data (glist, ft->crlf))))), charset))
+                        (GTK_BIN (g_list_nth_data (glist, ft->crlf))))), str))
       break;
   g_list_free (glist);
   if (ft->crlf < 0)
     ft->crlf = 0;
-  g_free (ft->charset);
-  ft->charset = g_strdup (gtk_entry_get_text
-                                    (GTK_ENTRY (GTK_COMBO (combo1)->entry)));
+  ft->charset = misc_charset_get_charset (MISC_CHARSET (charset));
   gtk_widget_destroy (dialog);
   reload_clicked_ok = TRUE;
 }
@@ -98,8 +96,7 @@ gboolean
 reload_dialog (FileType    *ft,
                const gchar *text)
 {
-  gchar **charset;
-  gint i, j;
+  gint i;
   GList *glist;
   GtkWidget *button1, *hbox, *vbox;
 
@@ -141,41 +138,24 @@ reload_dialog (FileType    *ft,
   glist = g_list_append (NULL, "CR+LF");
   glist = g_list_append (glist, "CR");
   glist = g_list_append (glist, "LF");
-  combo0 = gtk_combo_new();
-  gtk_combo_set_popdown_strings (GTK_COMBO (combo0), glist);
-  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo0)->entry),
+  combo = gtk_combo_new ();
+  gtk_combo_set_popdown_strings (GTK_COMBO (combo), glist);
+  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry),
                                             g_list_nth_data (glist, ft->crlf));
-  gtk_editable_set_editable (GTK_EDITABLE (GTK_COMBO (combo0)->entry), FALSE);
-  g_signal_connect (G_OBJECT (GTK_COMBO (combo0)->entry), "activate",
+  gtk_editable_set_editable (GTK_EDITABLE (GTK_COMBO (combo)->entry), FALSE);
+  g_signal_connect (G_OBJECT (GTK_COMBO (combo)->entry), "activate",
                                     G_CALLBACK (reload_dialog_activate), ft);
-  g_signal_connect (G_OBJECT (GTK_COMBO (combo0)->entry), "changed",
+  g_signal_connect (G_OBJECT (GTK_COMBO (combo)->entry), "changed",
                                 G_CALLBACK (reload_dialog_ok_sensitive), NULL);
-  glist = NULL;
+  /* ja:文字符号化方式 */
+  charset = misc_charset_new ();
   for (i = 0; i < ftnum; i++)
-    {
-      charset = g_strsplit (ftype[i].charset, ",", G_MAXINT);
-      for (j = 0; charset[j]; j++)
-        if (!g_list_find_custom (glist, charset[j],
-                                                (GCompareFunc)g_strcasecmp))
-          glist = g_list_insert_sorted (glist, g_strdup (charset[j]),
-                                                (GCompareFunc)g_strcasecmp);
-      g_strfreev (charset);
-    }
-  if (!g_list_find_custom (glist, ft->charset, (GCompareFunc)g_strcasecmp))
-    glist = g_list_insert_sorted (glist, g_strdup (ft->charset),
-                                                (GCompareFunc)g_strcasecmp);
-  combo1 = gtk_combo_new();
-  gtk_combo_set_popdown_strings (GTK_COMBO (combo1), glist);
-  gtk_combo_set_value_in_list (GTK_COMBO (combo1), FALSE, FALSE);
-  gtk_combo_set_case_sensitive (GTK_COMBO (combo1), TRUE);
-  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo1)->entry), ft->charset);
-  gtk_editable_select_region (GTK_EDITABLE (GTK_COMBO (combo1)->entry), 0, -1);
-  g_signal_connect (G_OBJECT (GTK_COMBO (combo1)->entry), "activate",
+    misc_charset_add_candidate (MISC_CHARSET (charset), ftype[i].charset);
+  misc_charset_set_charset (MISC_CHARSET (charset), ft->charset);
+  g_signal_connect (G_OBJECT (charset), "activate",
                                     G_CALLBACK (reload_dialog_activate), ft);
-  g_signal_connect (G_OBJECT (GTK_COMBO (combo1)->entry), "changed",
+  g_signal_connect (G_OBJECT (charset), "changed",
                                 G_CALLBACK (reload_dialog_ok_sensitive), NULL);
-  for (i = g_list_length (glist) - 1; i >= 0; i--)
-    g_free (g_list_nth_data (glist, i));
   /* ja:フレームとボックス */
   hbox = gtk_hbox_new (FALSE, SPACING);
   gtk_box_pack_end (GTK_BOX (hbox), button1, FALSE, FALSE, 0);
@@ -184,8 +164,8 @@ reload_dialog (FileType    *ft,
   gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);
   if (text)
     gtk_box_pack_start (GTK_BOX (vbox), gtk_label_new (text), FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), combo0, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), combo1, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), combo, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), charset, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), check0, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), check1, FALSE, FALSE, 0);
   gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
