@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include "sigmain.h"
 #include "misc/argument.h"
 #include "misc/misc.h"
+#include "misc/misc_mdi.h"
 
 
 /******************************************************************************
@@ -38,8 +39,6 @@ signal_style_set (GtkWidget *widget,
                   gpointer   user_data)
 {
   gint i;
-  GList *glist;
-  TmaidWindow *tmaid;
 
   /* ja:システム色 */
   style = gtk_widget_get_style (widget);
@@ -59,11 +58,11 @@ signal_style_set (GtkWidget *widget,
   system_font = pango_font_description_copy
                                     (pango_context_get_font_description
                                     (gtk_widget_get_pango_context (widget)));
-  glist = gtk_container_children (GTK_CONTAINER (notebook));
-  for (i = g_list_length (glist) - 1; i >= 0; i--)
+  for (i = misc_mdi_get_n_pages (MISC_MDI (mdi)) - 1; i >= 0; i--)
     {
-      tmaid = g_object_get_data (G_OBJECT (g_list_nth_data (glist, i)),
-                                                                "user_data");
+      TmaidWindow *tmaid;
+
+      tmaid = misc_mdi_get_data (MISC_MDI (mdi), i);
       if (tmaid->ft.syscol || !tmaid->ft.font_name)
         {
           if (!tmaid->ft.font_name)
@@ -93,7 +92,6 @@ signal_style_set (GtkWidget *widget,
           gtk_widget_draw (tmaid->drawing, NULL);/* ja:再描画 */
         }
     }
-  g_list_free (glist);
 }
 
 
@@ -102,16 +100,6 @@ signal_style_set (GtkWidget *widget,
 * ja:シグナル/イベント関数群(メニュー)                                        *
 *                                                                             *
 ******************************************************************************/
-void
-signal_activate_menu_window (GtkWidget *widget, GtkWidget *child)
-{
-  gtk_notebook_set_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_page_num (GTK_NOTEBOOK (notebook), child));
-  gtk_widget_grab_focus (((TmaidWindow *)
-                g_object_get_data (G_OBJECT (child), "user_data"))->drawing);
-}
-
-
 void
 signal_activate_menu_history (GtkWidget *widget,
                               gchar     *file)
@@ -130,39 +118,32 @@ signal_destroy_menu_history (GtkWidget *widget,
 
 /******************************************************************************
 *                                                                             *
-* ja:シグナル/イベント関数群(ノート)                                          *
+* ja:シグナル/イベント関数群(MDI)                                             *
 *                                                                             *
 ******************************************************************************/
 void
-signal_clicked (GtkWidget *widget,
-                GtkWidget *child)
+signal_close_window (GtkWidget   *widget,
+                     guint        page_num,
+                     TmaidWindow *tmaid)
 {
-  if (prompt_close ((TmaidWindow *)g_object_get_data (G_OBJECT (child),
-                                                                "user_data")))
-    gtk_notebook_remove_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_page_num (GTK_NOTEBOOK (notebook), child));
+  if (prompt_close (tmaid))
+    gtk_notebook_remove_page (GTK_NOTEBOOK (mdi), page_num);
 }
 
 
 void
-signal_switch_page (GtkNotebook     *notebook,
-                    GtkNotebookPage *page,
-                    gint             page_num,
-                    gpointer         user_data)
+signal_switch_window (GtkWidget   *widget,
+                      guint        page_num,
+                      TmaidWindow *tmaid)
 {
-  TmaidWindow *tmaid;
-
-  tmaid = g_object_get_data
-                    (G_OBJECT (gtk_notebook_get_nth_page (notebook, page_num)),
-                                                                "user_data");
   /* ja:メニュー表示 */
   set_menu_bar (tmaid);
 }
 
 
 void
-signal_destroy_notebook (GtkWidget *widget,
-                         gpointer   user_data)
+signal_destroy_mdi (GtkWidget *widget,
+                    gpointer   user_data)
 {
   gtk_timeout_remove (timer_id);
 }
@@ -286,22 +267,21 @@ signal_selection_received (GtkWidget        *widget,
         }
       if (text)
         {
+          gint page_num;
           GdkPoint select;
           TmaidWindow *tmaid;
           TmaidHistory *d;
 
-          tmaid = g_object_get_data(G_OBJECT
-                    (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
+          page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
+          tmaid = misc_mdi_get_data (MISC_MDI (mdi), page_num);
           select = tmaid->select;
           d = edit_operation (tmaid, text, g_strlen (text),
                             data->selection == GDK_SELECTION_CLIPBOARD, FALSE);
           d->next = tmaid->undo;
           tmaid->undo = d;
-          if (delete_list(&tmaid->redo) > 0 || !d->next || select.x >= 0)
-            set_menu_bar(tmaid);
-          tmaid_change_edit (tmaid, TRUE);
+          if (delete_list (&tmaid->redo) > 0 || !d->next || select.x >= 0)
+            set_menu_bar (tmaid);
+          misc_mdi_set_edited (MISC_MDI (mdi), page_num, TRUE);
           g_free (text);
         }
     }
@@ -392,16 +372,13 @@ signal_delete (GtkWidget *widget,
                gpointer   user_data)
 {
   gint page;
-  TmaidWindow *tmaid;
 
-  while ((page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook))) >= 0)
+  while ((page = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi))) >= 0)
     {
-      tmaid = g_object_get_data (G_OBJECT
-                (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), page)),
-                                                                "user_data");
-      if (!prompt_close (tmaid))
+      if (!prompt_close ((TmaidWindow *)
+                                    misc_mdi_get_data (MISC_MDI (mdi), page)))
         return TRUE;
-      gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page);
+      gtk_notebook_remove_page (GTK_NOTEBOOK (mdi), page);
     }
   return FALSE;
 }
