@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -43,11 +43,6 @@ void signal_style_set (GtkWidget *widget,
 * ja:シグナル/イベント関数群(メニュー)                                        *
 *                                                                             *
 ******************************************************************************/
-void
-signal_activate_menu_window (GtkWidget *widget,
-                             GtkWidget *child);
-
-
 void
 signal_activate_menu_history (GtkWidget *widget,
                               gchar     *file);
@@ -60,24 +55,24 @@ signal_destroy_menu_history (GtkWidget *widget,
 
 /******************************************************************************
 *                                                                             *
-* ja:シグナル/イベント関数群(ノート)                                          *
+* ja:シグナル/イベント関数群(MDI)                                             *
 *                                                                             *
 ******************************************************************************/
 void
-signal_clicked (GtkWidget *widget,
-                GtkWidget *child);
+signal_close_window (GtkWidget   *widget,
+                     guint        page_num,
+                     TmaidWindow *tmaid);
 
 
 void
-signal_switch_page (GtkNotebook     *notebook,
-                    GtkNotebookPage *page,
-                    gint             page_num,
-                    gpointer         user_data);
+signal_switch_window (GtkWidget   *widget,
+                      guint        page_num,
+                      TmaidWindow *tmaid);
 
 
 void
-signal_destroy_notebook (GtkWidget *widget,
-                         gpointer   user_data);
+signal_destroy_mdi (GtkWidget *widget,
+                    gpointer   user_data);
 
 
 /******************************************************************************
