@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 */
 #include <locale.h>
 #include "tmaid.h"
-#include "charlist.h"
 #include "command.h"
 #include "file.h"
 #include "find.h"
@@ -32,10 +31,13 @@
 #include "toolbar.h"
 #include "version.h"
 #include "misc/argument.h"
+#include "misc/charuty.h"
 #include "misc/fileio.h"
 #include "misc/license.h"
 #include "misc/misc_accel.h"
 #include "misc/misc_comm.h"
+#include "misc/misc_mdi.h"
+#include "misc/misc_winmenu.h"
 #include "misc/profile.h"
 
 
@@ -49,6 +51,9 @@ gchar *save_path = NULL;            /* ja:名前を付けて保存のパス */
 gchar *clipboard_text = NULL;       /* ja:クリップボードのテキスト */
 gint def_width, def_height;         /* ja:デフォルトのウインドウのサイズ */
 gint ftnum = 0;                     /* ja:ファイルタイプの数 */
+#if defined USE_GTK24 || defined G_OS_WIN32
+gint findex = 0;                    /* ja:ファイルタイプのフィルタ */
+#endif /* USE_GTK24 or G_OS_WIN32 */
 gint history = 4;                   /* ja:ファイルの履歴 */
 gint tcount = 0;                    /* ja:ファイルタイプのカウンタ */
 guint timer_id = 0;                 /* ja:タイマID */
@@ -58,7 +63,7 @@ GdkAtom atom_text;                  /* ja:テキスト */
 GdkAtom atom_compound;              /* ja:コンパウンド */
 GdkAtom atom_utf8;                  /* ja:UTF-8 */
 GtkWidget *window;                  /* ja:ウインドウ */
-GtkWidget *notebook;                /* ja:ノートブック */
+GtkWidget *mdi;                     /* en:MDI */
 GtkWidget *status;                  /* ja:ステータスウインドウ */
 GtkWidget *label_cursor;            /* ja:カーソル座標 */
 GtkPositionType n_pos = GTK_POS_TOP;/* ja:ノートブックの位置 */
@@ -102,9 +107,7 @@ main (int   argc,
   gint i, count = 0, init_line = 0;
   Profile *profile;
   TmaidWindow *tmaid;
-  GObject *accel;
-  GdkBitmap *mask;
-  GdkPixmap *pixmap;
+  GObject *accel, *winmenu;
   GtkAccelGroup *accel_group;
   GtkStyle *style;
   GtkWidget *menu_bar, *menu_shell, *menu_item;
@@ -199,6 +202,11 @@ main (int   argc,
       if (!profile_get_value(profile, "tmaid", "ftnum",
             (gpointer)&ftnum, sizeof (gint), PROFILE_VALUE_TYPE_INT))
         ftnum = 0;
+#if defined USE_GTK24 || defined G_OS_WIN32
+      if (!profile_get_value(profile, "tmaid", "findex",
+            (gpointer)&findex, sizeof (gint), PROFILE_VALUE_TYPE_INT))
+        findex = 0;
+#endif /* defined USE_GTK24 or defined G_OS_WIN32 */
       if (!profile_get_value (profile, "tmaid","history",
             (gpointer)&history, sizeof (gint), PROFILE_VALUE_TYPE_INT))
         history = 4;
@@ -434,7 +442,7 @@ main (int   argc,
           ftype[i].charset = profile_get_string (profile, section, "charset");
           if (ftype[i].charset)
             {
-              tmp = charlist_strip (ftype[i].charset);
+              tmp = charuty_strip (ftype[i].charset);
               g_free (ftype[i].charset);
               ftype[i].charset = tmp;
             }
@@ -579,14 +587,16 @@ TMAID_COPYRIGHT_NOTICE"\n"
                                     G_CALLBACK (signal_selection_get), NULL);
   g_signal_connect (G_OBJECT (window), "selection-received",
                                 G_CALLBACK (signal_selection_received), NULL);
-  /* ja:ノートブック */
-  notebook = gtk_notebook_new ();
-  gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
-  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), n_pos);
-  g_signal_connect (G_OBJECT (notebook), "switch-page",
-                                G_CALLBACK (signal_switch_page), NULL);
-  g_signal_connect (G_OBJECT (notebook), "destroy",
-                                G_CALLBACK (signal_destroy_notebook), NULL);
+  /* en:MDI */
+  mdi = misc_mdi_new ();
+  gtk_notebook_set_scrollable (GTK_NOTEBOOK (mdi), TRUE);
+  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (mdi), n_pos);
+  g_signal_connect (G_OBJECT (mdi), "close-window",
+                                G_CALLBACK (signal_close_window), NULL);
+  g_signal_connect (G_OBJECT (mdi), "switch-window",
+                                G_CALLBACK (signal_switch_window), NULL);
+  g_signal_connect (GTK_OBJECT (mdi), "destroy",
+                                G_CALLBACK (signal_destroy_mdi), NULL);
   /* ja:ポップアップ */
   accel_group = gtk_accel_group_new ();
   ifactory_popup = gtk_item_factory_new (GTK_TYPE_MENU, "<main>", accel_group);
@@ -605,6 +615,11 @@ TMAID_COPYRIGHT_NOTICE"\n"
                         sizeof (menu_items) / sizeof (GtkItemFactoryEntry),
                                                             menu_items, NULL);
   menu_bar = gtk_item_factory_get_widget (ifactory_menu, "<main>");
+  /* ja:ウインドウメニュー */
+  winmenu = misc_winmenu_new ();
+  misc_winmenu_set_mdi (MISC_WINMENU (winmenu), MISC_MDI (mdi));
+  misc_winmenu_set_menu (MISC_WINMENU (winmenu),
+            gtk_item_factory_get_widget (ifactory_menu, "<main>/Window"), 0);
   /* ja:アクセルグループ */
   accel = misc_accel_new ();
   misc_accel_set_window (MISC_ACCEL (accel), GTK_WINDOW (window));
@@ -617,14 +632,17 @@ TMAID_COPYRIGHT_NOTICE"\n"
   for (i = 0; toolbar_items[i].name != (gpointer)-1; i++)
     if (toolbar_items[i].name)
       {
+        GdkBitmap *mask;
+        GdkPixmap *pixmap;
+
         pixmap = gdk_pixmap_create_from_xpm_d (window->window,
                                             &mask, NULL, toolbar_items[i].xpm);
         toolbar_items[i].item = gtk_toolbar_append_item
-                        (GTK_TOOLBAR (toolbar),
-                        NULL, _(toolbar_items[i].name), NULL,
-                        gtk_pixmap_new (pixmap, mask),
-                        GTK_SIGNAL_FUNC (toolbar_items[i].callback),
-                        toolbar_items[i].user_data);
+                                (GTK_TOOLBAR (toolbar),
+                                NULL, _(toolbar_items[i].name), NULL,
+                                gtk_pixmap_new (pixmap, mask),
+                                GTK_SIGNAL_FUNC (toolbar_items[i].callback),
+                                toolbar_items[i].user_data);
       }
     else
       {
@@ -653,7 +671,7 @@ TMAID_COPYRIGHT_NOTICE"\n"
   gtk_container_set_border_width (GTK_CONTAINER (handle), 2);
   gtk_container_add (GTK_CONTAINER (handle), toolbar);
   gtk_box_pack_start (GTK_BOX (vbox), handle, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), notebook, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), mdi, TRUE, TRUE, 0);
   gtk_table_attach (GTK_TABLE (table), status, 0, 4, 0, 1,
             GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
   gtk_table_attach (GTK_TABLE (table), frame, 4, 5, 0, 1,
@@ -679,7 +697,7 @@ TMAID_COPYRIGHT_NOTICE"\n"
     gtk_menu_insert (GTK_MENU (menu_shell), gtk_menu_item_new (),
                                                             i + MENUFILE - 1);
   /* ja:タイマ */
-  timer_id = gtk_timeout_add (500, signal_timeout, NULL);
+  timer_id = g_timeout_add (500, signal_timeout, NULL);
 
   /* ja:メニュー表示 */
   set_menu_bar (NULL);
@@ -744,6 +762,10 @@ TMAID_COPYRIGHT_NOTICE"\n"
             (gpointer)&license, sizeof (gboolean), PROFILE_VALUE_TYPE_BOOL);
       profile_set_value (profile, "tmaid", "ftnum",
             (gpointer)&ftnum, sizeof (gint), PROFILE_VALUE_TYPE_INT);
+#if defined USE_GTK24 || defined G_OS_WIN32
+      profile_set_value (profile, "tmaid", "findex",
+            (gpointer)&findex, sizeof (gint), PROFILE_VALUE_TYPE_INT);
+#endif /* USE_GTK24 or G_OS_WIN32 */
       profile_set_value (profile, "tmaid", "history",
             (gpointer)&history, sizeof (gint), PROFILE_VALUE_TYPE_INT);
       profile_set_value (profile, "tmaid", "newfile",
